@@ -12,12 +12,12 @@
 # configuration option turned on, unless proper preconfiguration of
 # the chroot environment has been taken care of!
 
-SOURCE="$1"
-DEST="$2"
-MEMLIMIT="$3"
+DEST="$1" ; shift
+MEMLIMIT="$1" ; shift
+MAINSOURCE="$1"
 MAINCLASS=""
 
-# Amount of memory reserved for the Java virtual machine in KB. The
+# Amount of memory reserved for the Java virtual machine in kB. The
 # default below is just above the maximum memory usage of current
 # versions of the jvm, but might need increasing in some cases.
 MEMRESERVED=300000
@@ -25,7 +25,7 @@ MEMRESERVED=300000
 TMPFILE=`mktemp /tmp/domjudge_javac_output.XXXXXX` || exit 1
 
 # Byte-compile:
-javac -d . "$SOURCE" 2> "$TMPFILE"
+javac -d . "$@" 2> "$TMPFILE"
 EXITCODE=$?
 if [ "$EXITCODE" -ne 0 ]; then
 	# Let's see if should have named the .java differently
@@ -36,8 +36,8 @@ if [ "$EXITCODE" -ne 0 ]; then
 		exit $EXITCODE
 	fi
 	rm -f $TMPFILE
-	echo "Info: renaming source to '$PUBLICCLASS.java'"
-	mv "$SOURCE" "$PUBLICCLASS.java"
+	echo "Info: renaming main source '$MAINSOURCE' to '$PUBLICCLASS.java'"
+	mv "$MAINSOURCE" "$PUBLICCLASS.java"
 	javac -d . "$PUBLICCLASS.java"
 	EXITCODE=$?
 	[ "$EXITCODE" -ne 0 ] && exit $EXITCODE
@@ -49,7 +49,7 @@ rm -f $TMPFILE
 for cn in $(find * -type f -regex '^.*\.class$' \
 		| sed -e 's/\.class$//' -e 's/\//./'); do
 	javap -public "$cn" \
-	| grep -q 'public static void main(java.lang.String\[\])' \
+	| egrep -q 'public static void main\(java.lang.String(\[\]|\.\.\.)\)' \
 	&& {
 		if [ -n "$MAINCLASS" ]; then
 			echo "Warning: found another 'main' in '$cn'"
@@ -76,6 +76,11 @@ cat > $DEST <<EOF
 #!/bin/sh
 # Generated shell-script to execute java interpreter on source.
 
+# Detect dirname and change dir to prevent class not found errors.
+if [ "\${0%/*}" != "\$0" ]; then
+	cd "\${0%/*}"
+fi
+
 exec java -Xrs -Xss8m -Xmx${MEMLIMITJAVA}k $MAINCLASS
 EOF
 
