@@ -1,12 +1,12 @@
 /*
    runguard -- run command with restrictions.
-   Copyright (C) 2004-2009 Jaap Eldering (eldering@a-eskwadraat.nl).
+   Copyright (C) 2004-2012 Jaap Eldering (eldering@a-eskwadraat.nl).
+
+   Multiple minor improvements ported from the DOMjudge-ETH tree.
 
    Based on an idea from the timeout program, written by Wietse Venema
    as part of The Coroner's Toolkit.
 
-   $Id: runguard.c 3602 2011-08-17 12:57:40Z eldering $
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
@@ -31,7 +31,7 @@
    The stdin and stdout streams are passed to the command and runguard
    does not read or write to these. Error and verbose messages from
    runguard are by default written to stderr, hence mixed with stderr
-   output of the command.
+   output of the command, unless that is optionally redirected to file.
 
    The command and its children are sent a SIGTERM after the runtime
    has passed, followed by a SIGKILL after 'killdelay'.
@@ -45,9 +45,13 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/param.h>
+#include <sys/select.h>
+#include <sys/stat.h>
 #include <sys/time.h>
+#include <sys/times.h>
 #include <sys/resource.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -65,9 +69,18 @@
 #include "runguard-config.h"
 
 #define PROGRAM "runguard"
-#define VERSION "$Rev: 3602 $"
+#define VERSION DOMJUDGE_VERSION "/" REVISION
 #define AUTHORS "Jaap Eldering"
 
+#define max(x,y) ((x) > (y) ? (x) : (y))
+
+/* Array indices for input/output file descriptors as used by pipe() */
+#define PIPE_IN  1
+#define PIPE_OUT 0
+
+#define BUF_SIZE 4*1024
+char buf[BUF_SIZE];
+
 const struct timespec killdelay = { 0, 100000000L }; /* 0.1 seconds */
 
 extern int errno;
@@ -82,15 +95,23 @@ char  *progname;
 char  *cmdname;
 char **cmdargs;
 char  *rootdir;
-char  *outputfilename;
+char  *stdoutfilename;
+char  *stderrfilename;
+char  *exitfilename;
+char  *timefilename;
 
 int runuid;
 int rungid;
 int use_root;
 int use_time;
+int use_cputime;
 int use_user;
 int use_group;
-int use_output;
+int redir_stdout;
+int redir_stderr;
+int limit_streamsize;
+int outputexit;
+int outputtime;
 int no_coredump;
 int be_verbose;
 int be_quiet;
@@ -98,33 +119,50 @@ int show_help;
 int show_version;
 
 unsigned long runtime; /* in microseconds */
-rlim_t cputime;
+unsigned long cputime;
 rlim_t memsize;
 rlim_t filesize;
 rlim_t nproc;
+size_t streamsize;
 
 pid_t child_pid;
 
+static volatile sig_atomic_t received_SIGCHLD = 0;
+
+FILE *child_stdout;
+FILE *child_stderr;
+int child_pipefd[3][2];
+int child_redirfd[3];
+
 struct timeval starttime, endtime;
+struct tms startticks, endticks;
 
 struct option const long_opts[] = {
-	{"root",    required_argument, NULL,         'r'},
-	{"user",    required_argument, NULL,         'u'},
-	{"group",   required_argument, NULL,         'g'},
-	{"time",    required_argument, NULL,         't'},
-	{"cputime", required_argument, NULL,         'C'},
-	{"memsize", required_argument, NULL,         'm'},
-	{"filesize",required_argument, NULL,         'f'},
-	{"nproc",   required_argument, NULL,         'p'},
-	{"no-core", no_argument,       NULL,         'c'},
-	{"output",  required_argument, NULL,         'o'},
-	{"verbose", no_argument,       NULL,         'v'},
-	{"quiet",   no_argument,       NULL,         'q'},
-	{"help",    no_argument,       &show_help,    1 },
-	{"version", no_argument,       &show_version, 1 },
-	{ NULL,     0,                 NULL,          0 }
+	{"root",       required_argument, NULL,         'r'},
+	{"user",       required_argument, NULL,         'u'},
+	{"group",      required_argument, NULL,         'g'},
+	{"time",       required_argument, NULL,         't'},
+	{"cputime",    required_argument, NULL,         'C'},
+	{"memsize",    required_argument, NULL,         'm'},
+	{"filesize",   required_argument, NULL,         'f'},
+	{"nproc",      required_argument, NULL,         'p'},
+	{"no-core",    no_argument,       NULL,         'c'},
+	{"stdout",     required_argument, NULL,         'o'},
+	{"stderr",     required_argument, NULL,         'e'},
+	{"streamsize", required_argument, NULL,         's'},
+	{"outexit",    required_argument, NULL,         'E'},
+	{"outtime",    required_argument, NULL,         'T'},
+	{"verbose",    no_argument,       NULL,         'v'},
+	{"quiet",      no_argument,       NULL,         'q'},
+	{"help",       no_argument,       &show_help,    1 },
+	{"version",    no_argument,       &show_version, 1 },
+	{ NULL,        0,                 NULL,          0 }
 };
 
+void warning(   const char *, ...) __attribute__((format (printf, 1, 2)));
+void verbose(   const char *, ...) __attribute__((format (printf, 1, 2)));
+void error(int, const char *, ...) __attribute__((format (printf, 2, 3)));
+
 void warning(const char *format, ...)
 {
 	va_list ap;
@@ -195,21 +233,26 @@ Usage: %s [OPTION]... COMMAND...\n\
 Run COMMAND with restrictions.\n\
 \n", progname);
 	printf("\
-  -r, --root=ROOT      run COMMAND with root directory set to ROOT\n\
-  -u, --user=USER      run COMMAND as user with username or ID USER\n\
-  -g, --group=GROUP    run COMMAND under group with name or ID GROUP\n\
-  -t, --time=TIME      kill COMMAND if still running after TIME seconds (float)\n\
-  -C, --cputime=TIME   set maximum CPU time to TIME seconds (integer)\n\
-  -m, --memsize=SIZE   set all (total, stack, etc) memory limits to SIZE kB\n\
-  -f, --filesize=SIZE  set maximum created filesize to SIZE kB\n");
+  -r, --root=ROOT        run COMMAND with root directory set to ROOT\n\
+  -u, --user=USER        run COMMAND as user with username or ID USER\n\
+  -g, --group=GROUP      run COMMAND under group with name or ID GROUP\n\
+  -t, --time=TIME        kill COMMAND after TIME seconds (float)\n\
+  -C, --cputime=TIME     set maximum CPU time to TIME seconds (float)\n\
+  -m, --memsize=SIZE     set all (total, stack, etc) memory limits to SIZE kB\n\
+  -f, --filesize=SIZE    set maximum created filesize to SIZE kB;\n");
 	printf("\
-  -p, --nproc=N        set maximum no. processes to N\n\
-  -c, --no-core        disable core dumps\n\
-  -o, --output=FILE    write actual runtime to FILE\n\
-  -v, --verbose        display some extra warnings and information\n\
-  -q, --quiet          suppress all warnings and verbose output\n\
-      --help           display this help and exit\n\
-      --version        output version information and exit\n");
+  -p, --nproc=N          set maximum no. processes to N\n\
+  -c, --no-core          disable core dumps\n\
+  -o, --stdout=FILE      redirect COMMAND stdout output to FILE\n\
+  -e, --stderr=FILE      redirect COMMAND stderr output to FILE\n\
+  -s, --streamsize=SIZE  truncate COMMAND stdout/stderr streams at SIZE kB\n\
+  -E, --outexit=FILE     write COMMAND exitcode to FILE\n\
+  -T, --outtime=FILE     write COMMAND runtime to FILE\n");
+	printf("\
+  -v, --verbose          display some extra warnings and information\n\
+  -q, --quiet            suppress all warnings and verbose output\n\
+      --help             display this help and exit\n\
+      --version          output version information and exit\n");
 	printf("\n\
 Note that root privileges are needed for the `root' and `user' options.\n\
 When run setuid without the `user' option, the user ID is set to the\n\
@@ -217,29 +260,49 @@ real user ID.\n");
 	exit(0);
 }
 
-void outputtime()
+void output_exit_time(int exitcode, double timediff)
 {
 	FILE  *outputfile;
-	double timediff; /* in seconds */
+	double userdiff, sysdiff;
+	unsigned long ticks_per_second = sysconf(_SC_CLK_TCK);
 
-	if ( gettimeofday(&endtime,NULL) ) error(errno,"getting time");
+	verbose("command exited with exitcode %d",exitcode);
 
-	timediff = (endtime.tv_sec  - starttime.tv_sec ) +
-	           (endtime.tv_usec - starttime.tv_usec)*1E-6;
+	if ( outputexit ) {
+		verbose("writing exitcode to file `%s'",exitfilename);
 
-	verbose("runtime is %.3f seconds",timediff);
-
-	if ( use_output ) {
-		verbose("writing runtime to file `%s'",outputfilename);
-
-		if ( (outputfile = fopen(outputfilename,"w"))==NULL ) {
-			error(errno,"cannot open `%s'",outputfilename);
+		if ( (outputfile = fopen(exitfilename,"w"))==NULL ) {
+			error(errno,"cannot open `%s'",exitfilename);
 		}
-		if ( fprintf(outputfile,"%.3f\n",timediff)==0 ) {
-			error(0,"cannot write to file `%s'",outputfile);
+		if ( fprintf(outputfile,"%d\n",exitcode)==0 ) {
+			error(0,"cannot write to file `%s'",exitfilename);
 		}
 		if ( fclose(outputfile) ) {
-			error(errno,"closing file `%s'",outputfilename);
+			error(errno,"closing file `%s'",exitfilename);
+		}
+	}
+
+	userdiff = (double)(endticks.tms_cutime - startticks.tms_cutime) / ticks_per_second;
+	sysdiff  = (double)(endticks.tms_cstime - startticks.tms_cstime) / ticks_per_second;
+
+	verbose("runtime is %.3f seconds real, %.3f user, %.3f sys\n",
+	        timediff, userdiff, sysdiff);
+
+	if ( use_cputime && (userdiff+sysdiff) * 1000000 > cputime ) {
+		warning("timelimit exceeded (cpu time)");
+	}
+
+	if ( outputtime ) {
+		verbose("writing runtime to file `%s'",timefilename);
+
+		if ( (outputfile = fopen(timefilename,"w"))==NULL ) {
+			error(errno,"cannot open `%s'",timefilename);
+		}
+		if ( fprintf(outputfile,"%.3f\n",userdiff+sysdiff)==0 ) {
+			error(0,"cannot write to file `%s'",timefilename);
+		}
+		if ( fclose(outputfile) ) {
+			error(errno,"closing file `%s'",timefilename);
 		}
 	}
 }
@@ -254,7 +317,7 @@ void terminate(int sig)
 	if ( sigaction(SIGALRM,&sigact,NULL)!=0 ) warning("error restoring signal handler");
 
 	if ( sig==SIGALRM ) {
-		warning("timelimit reached: aborting command");
+		warning("timelimit exceeded (wall time): aborting command");
 	} else {
 		warning("received signal %d: aborting command",sig);
 	}
@@ -269,6 +332,14 @@ void terminate(int sig)
 
 	verbose("sending SIGKILL");
 	if ( kill(-child_pid,SIGKILL)!=0 ) error(errno,"sending SIGKILL to command");
+
+	/* Wait another while to make sure the process is killed by now. */
+	nanosleep(&killdelay,NULL);
+}
+
+static void child_handler(int sig)
+{
+	received_SIGCHLD = 1;
 }
 
 int userid(char *name)
@@ -334,31 +405,32 @@ void setrestrictions()
 		} \
 	}
 
-	if ( cputime!=RLIM_INFINITY ) {
-		verbose("setting CPU-time limit to %d seconds",(int)cputime);
+	if ( use_cputime ) {
+		rlim_t cputime_limit = (rlim_t)(cputime/1000000 + 1);
+		verbose("setting CPU-time limit to %d seconds",(int)cputime_limit);
+		lim.rlim_cur = lim.rlim_max = cputime_limit;
+		setlim(CPU);
 	}
-	lim.rlim_cur = lim.rlim_max = cputime;
-	setlim(CPU);
 
 	if ( memsize!=RLIM_INFINITY ) {
 		verbose("setting memory limits to %d bytes",(int)memsize);
+		lim.rlim_cur = lim.rlim_max = memsize;
+		setlim(AS);
+		setlim(DATA);
+		setlim(STACK);
 	}
-	lim.rlim_cur = lim.rlim_max = memsize;
-	setlim(AS);
-	setlim(DATA);
-	setlim(STACK);
 
 	if ( filesize!=RLIM_INFINITY ) {
 		verbose("setting filesize limit to %d bytes",(int)filesize);
+		lim.rlim_cur = lim.rlim_max = filesize;
+		setlim(FSIZE);
 	}
-	lim.rlim_cur = lim.rlim_max = filesize;
-	setlim(FSIZE);
 
 	if ( nproc!=RLIM_INFINITY ) {
 		verbose("setting process limit to %d",(int)nproc);
+		lim.rlim_cur = lim.rlim_max = nproc;
+		setlim(NPROC);
 	}
-	lim.rlim_cur = lim.rlim_max = nproc;
-	setlim(NPROC);
 
 #undef setlim
 
@@ -414,19 +486,25 @@ void setrestrictions()
 		if ( setuid(getuid()) ) error(errno,"cannot reset real user ID");
 		verbose("reset user ID to `%d' for command",getuid());
 	}
-	if ( geteuid()==0 || getuid()==0 ) error(0,"root privileges not dropped");
+	if ( geteuid()==0 || getuid()==0 ) error(0,"root privileges not dropped. Do not run judgedaemon as root.");
 }
 
 int main(int argc, char **argv)
 {
-	sigset_t sigmask;
+	sigset_t sigmask, emptymask;
+	fd_set readfds;
 	pid_t pid;
+	int   i, r, nfds;
 	int   status;
 	int   exitcode;
 	char *valid_users;
 	char *ptr;
 	int   opt;
 	double runtime_d;
+	double cputime_d;
+	double timediff;
+	size_t data_passed[3];
+	ssize_t nread, nwritten;
 
 	struct itimerval itimer;
 	struct sigaction sigact;
@@ -434,12 +512,13 @@ int main(int argc, char **argv)
 	progname = argv[0];
 
 	/* Parse command-line options */
-	use_root = use_time = use_user = use_output = no_coredump = 0;
+	use_root = use_time = use_cputime = use_user = outputexit = outputtime = no_coredump = 0;
 	cputime = memsize = filesize = nproc = RLIM_INFINITY;
+	redir_stdout = redir_stderr = limit_streamsize = 0;
 	be_verbose = be_quiet = 0;
 	show_help = show_version = 0;
 	opterr = 0;
-	while ( (opt = getopt_long(argc,argv,"+r:u:g:t:C:m:f:p:co:vq",long_opts,(int *) 0))!=-1 ) {
+	while ( (opt = getopt_long(argc,argv,"+r:u:g:t:C:m:f:p:co:e:s:E:T:vq",long_opts,(int *) 0))!=-1 ) {
 		switch ( opt ) {
 		case 0:   /* long-only option */
 			break;
@@ -470,7 +549,12 @@ int main(int argc, char **argv)
 			runtime = (unsigned long)(runtime_d*1E6);
 			break;
 		case 'C': /* CPU time option */
-			cputime = (rlim_t) readoptarg("CPU-time limit",1,LONG_MAX);
+			use_cputime = 1;
+			cputime_d = strtod(optarg,&ptr);
+			if ( errno || *ptr!='\0' || !finite(cputime_d) || cputime_d<=0 ) {
+				error(errno,"invalid cputime specified: `%s'",optarg);
+			}
+			cputime = (int)(cputime_d*1E6);
 			break;
 		case 'm': /* memsize option */
 			memsize = (rlim_t) readoptarg("memory limit",1,LONG_MAX);
@@ -496,9 +580,31 @@ int main(int argc, char **argv)
 		case 'c': /* no-core option */
 			no_coredump = 1;
 			break;
-		case 'o': /* output option */
-			use_output = 1;
-			outputfilename = strdup(optarg);
+		case 'o': /* stdout option */
+			redir_stdout = 1;
+			stdoutfilename = strdup(optarg);
+			break;
+		case 'e': /* stderr option */
+			redir_stderr = 1;
+			stderrfilename = strdup(optarg);
+			break;
+		case 's': /* streamsize option */
+			limit_streamsize = 1;
+			streamsize = (size_t) readoptarg("streamsize limit",0,LONG_MAX);
+			/* Convert limit from kB to bytes and check for overflow */
+			if ( streamsize!=(streamsize*1024)/1024 ) {
+				streamsize = (size_t) LONG_MAX;
+			} else {
+				streamsize *= 1024;
+			}
+			break;
+		case 'E': /* outputexit option */
+			outputexit = 1;
+			exitfilename = strdup(optarg);
+			break;
+		case 'T': /* outputtime option */
+			outputtime = 1;
+			timefilename = strdup(optarg);
 			break;
 		case 'v': /* verbose option */
 			be_verbose = 1;
@@ -534,10 +640,44 @@ int main(int argc, char **argv)
 		if ( ptr==NULL || runuid<=0 ) error(0,"illegal user specified: %d",runuid);
 	}
 
+	/* Setup pipes connecting to child stdout/err streams (ignore stdin). */
+	for(i=1; i<=2; i++) {
+		if ( pipe(child_pipefd[i])!=0 ) error(errno,"creating pipe for fd %d",i);
+	}
+
+	if ( sigemptyset(&emptymask)!=0 ) error(errno,"creating empty signal mask");
+
+	/* unmask all signals, except SIGCHLD: detected in pselect() below */
+	sigmask = emptymask;
+	if ( sigaddset(&sigmask, SIGCHLD)!=0 ) error(errno,"setting signal mask");
+	if ( sigprocmask(SIG_SETMASK, &sigmask, NULL)!=0 ) {
+		error(errno,"unmasking signals");
+	}
+
+	/* Construct signal handler for SIGCHLD detection in pselect(). */
+	received_SIGCHLD = 0;
+	sigact.sa_handler = child_handler;
+	sigact.sa_flags   = 0;
+	sigact.sa_mask    = emptymask;
+	if ( sigaction(SIGCHLD,&sigact,NULL)!=0 ) {
+		error(errno,"installing signal handler");
+	}
+
 	switch ( child_pid = fork() ) {
 	case -1: /* error */
 		error(errno,"cannot fork");
 	case  0: /* run controlled command */
+		/* Connect pipes to command (stdin/)stdout/stderr and close unneeded fd's */
+		for(i=1; i<=2; i++) {
+			if ( dup2(child_pipefd[i][PIPE_IN],i)<0 ) {
+				error(errno,"redirecting child fd %d",i);
+			}
+			if ( close(child_pipefd[i][PIPE_IN] )!=0 ||
+			     close(child_pipefd[i][PIPE_OUT])!=0 ) {
+				error(errno,"closing pipe for fd %d",i);
+			}
+		}
+
 		/* Run the command in a separate process group so that the command
 		   and all its children can be killed off with one signal. */
 		if ( setsid()==-1 ) error(errno,"setsid failed");
@@ -550,7 +690,6 @@ int main(int argc, char **argv)
 		error(errno,"cannot start `%s'",cmdname);
 
 	default: /* become watchdog */
-
 		/* Shed privileges, only if not using a separate child uid,
 		   because in that case we may need root privileges to kill
 		   the child process. Do not use Linux specific setresuid()
@@ -562,14 +701,36 @@ int main(int argc, char **argv)
 
 		if ( gettimeofday(&starttime,NULL) ) error(errno,"getting time");
 
-		/* unmask all signals */
-		if ( sigemptyset(&sigmask)!=0 ) error(errno,"creating signal mask");
-		if ( sigprocmask(SIG_SETMASK, &sigmask, NULL)!=0 ) {
-			error(errno,"unmasking signals");
+		/* Close unused file descriptors */
+		for(i=1; i<=2; i++) {
+			if ( close(child_pipefd[i][PIPE_IN])!=0 ) {
+				error(errno,"closing pipe for fd %i",i);
+			}
 		}
 
+		/* Redirect child stdout/stderr to file */
+		for(i=1; i<=2; i++) {
+			child_redirfd[i] = i; /* Default: no redirects */
+			data_passed[i] = 0; /* Reset data counters */
+		}
+		if ( redir_stdout ) {
+			child_redirfd[STDOUT_FILENO] = creat(stdoutfilename, S_IRUSR | S_IWUSR);
+			if ( child_redirfd[STDOUT_FILENO]<0 ) {
+				error(errno,"opening file '%s'",stdoutfilename);
+			}
+		}
+		if ( redir_stderr ) {
+			child_redirfd[STDERR_FILENO] = creat(stderrfilename, S_IRUSR | S_IWUSR);
+			if ( child_redirfd[STDERR_FILENO]<0 ) {
+				error(errno,"opening file '%s'",stderrfilename);
+			}
+		}
+
+		if ( sigemptyset(&emptymask)!=0 ) error(errno,"creating empty signal mask");
+
 		/* Construct one-time signal handler to terminate() for TERM
 		   and ALRM signals. */
+		sigmask = emptymask;
 		if ( sigaddset(&sigmask,SIGALRM)!=0 ||
 		     sigaddset(&sigmask,SIGTERM)!=0 ) error(errno,"setting signal mask");
 
@@ -600,27 +761,88 @@ int main(int argc, char **argv)
 			verbose("using timelimit of %.3f seconds",runtime*1E-6);
 		}
 
-		/* Wait for the child command to finish */
-		while ( (pid = wait(&status))!=-1 && pid!=child_pid );
-		if ( pid!=child_pid ) error(errno,"waiting on child");
+		if ( times(&startticks)==(clock_t) -1 ) {
+			error(errno,"getting start clock ticks");
+		}
 
-		/* Drop root before writing to output file. */
-		if ( setuid(getuid())!=0 ) error(errno,"dropping root privileges");
+		/* Wait for child data or exit. */
+		while ( 1 ) {
 
-		outputtime();
+			FD_ZERO(&readfds);
+			nfds = -1;
+			for(i=1; i<=2; i++) {
+				if ( child_pipefd[i][PIPE_OUT]>=0 ) {
+					FD_SET(child_pipefd[i][PIPE_OUT],&readfds);
+					nfds = max(nfds,child_pipefd[i][PIPE_OUT]);
+				}
+			}
+
+			r = pselect(nfds+1, &readfds, NULL, NULL, NULL, &emptymask);
+			if ( r==-1 && errno!=EINTR ) error(errno,"waiting for child data");
+
+			if ( received_SIGCHLD ) {
+				if ( (pid = wait(&status))<0 ) error(errno,"waiting on child");
+				if ( pid==child_pid ) break;
+			}
+
+			/* Check to see if data is available and pass it on */
+			for(i=1; i<=2; i++) {
+				if ( child_pipefd[i][PIPE_OUT] != -1 && FD_ISSET(child_pipefd[i][PIPE_OUT],&readfds) ) {
+					nread = read(child_pipefd[i][PIPE_OUT], buf, BUF_SIZE);
+					if ( nread==-1 ) error(errno,"reading child fd %d",i);
+					if ( nread==0 ) {
+						/* EOF detected: close fd and indicate this with -1 */
+						if ( close(child_pipefd[i][PIPE_OUT])!=0 ) {
+							error(errno,"closing pipe for fd %d",i);
+						}
+						child_pipefd[i][PIPE_OUT] = -1;
+						continue;
+					}
+					if ( limit_streamsize && data_passed[i]+nread>=streamsize ) {
+						if ( data_passed[i]<streamsize ) {
+							verbose("child fd %d limit reached",i);
+						}
+						nread = streamsize - data_passed[i];
+					}
+					nwritten = write(child_redirfd[i], buf, nread);
+					if ( nwritten==-1 ) error(errno,"writing child fd %d",i);
+					data_passed[i] += nwritten;
+				}
+			}
+		}
+
+		if ( times(&endticks)==(clock_t) -1 ) {
+			error(errno,"getting end clock ticks");
+		}
+
+		if ( gettimeofday(&endtime,NULL) ) error(errno,"getting time");
+
+		timediff = (endtime.tv_sec  - starttime.tv_sec ) +
+		           (endtime.tv_usec - starttime.tv_usec)*1E-6;
 
 		/* Test whether command has finished abnormally */
+		exitcode = 0;
 		if ( ! WIFEXITED(status) ) {
 			if ( WIFSIGNALED(status) ) {
 				warning("command terminated with signal %d",WTERMSIG(status));
-				return 128+WTERMSIG(status);
+				exitcode = 128+WTERMSIG(status);
+			} else
+			if ( WIFSTOPPED(status) ) {
+				warning("command stopped with signal %d",WSTOPSIG(status));
+				exitcode = 128+WSTOPSIG(status);
+			} else {
+				error(0,"command exit status unknown: %d",status);
 			}
-			error(0,"command exit status unknown: %d",status);
+		} else {
+			exitcode = WEXITSTATUS(status);
 		}
 
+		/* Drop root before writing to output file(s). */
+		if ( setuid(getuid())!=0 ) error(errno,"dropping root privileges");
+
+		output_exit_time(exitcode, timediff);
+
 		/* Return the exitstatus of the command */
-		exitcode = WEXITSTATUS(status);
-		if ( exitcode!=0 ) verbose("command exited with exitcode %d",exitcode);
 		return exitcode;
 	}
 
