@@ -2,8 +2,6 @@
 /**
  * Miscellaneous helper functions
  *
- * $Id: lib.misc.php 3575 2011-07-30 11:28:24Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -14,6 +12,9 @@ define('MYSQL_DATETIME_FORMAT', '%Y-%m-%d %H:%M:%S');
 /** Perl regex class of allowed characters in identifier strings. */
 define('IDENTIFIER_CHARS', '[a-zA-Z0-9_-]');
 
+/** Perl regex of allowed filenames. */
+define('FILENAME_REGEX', '/^[a-zA-Z0-9][a-zA-Z0-9_\.-]*$/');
+
 /**
  * helperfunction to read all contents from a file.
  * If $sizelimit is true (default), then only limit this to
@@ -29,13 +30,8 @@ function getFileContents($filename, $sizelimit = true) {
 	}
 
 	if ( $sizelimit && filesize($filename) > 50000 ) {
-		// When our lowest supported PHP version >= 5.1.0, we can just use
-		// file_get_contents() with the maxlen parameter.
-		$fh = fopen($filename,'r');
-		if ( ! $fh ) error("Could not open $filename for reading");
-		$ret = fread($fh, 50000) . "\n[output truncated after 50,000 B]\n";
-		fclose($fh);
-		return $ret;
+		return file_get_contents($filename, FALSE, NULL, -1, 50000)
+			. "\n[output truncated after 50,000 B]\n";
 	}
 
 	return file_get_contents($filename);
@@ -54,10 +50,11 @@ function getCurContest($fulldata = FALSE) {
 	               WHERE enabled = 1 AND activatetime <= NOW()
 	               ORDER BY activatetime DESC LIMIT 1');
 
-	if ($now == NULL)
-		return FALSE;
-	else
-		return ( $fulldata ? $now : $now['cid'] );
+	if ( $now == NULL ) return FALSE;
+
+	if ( !$fulldata ) return $now['cid'];
+
+	return $now;
 }
 
 /**
@@ -77,6 +74,22 @@ function problemVisible($probid)
 	              $cdata['cid'], $probid) !== NULL;
 }
 
+/**
+ * Calculate contest time from wall-clock time.
+ * Returns time since contest start in seconds.
+ * This function is currently a stub around timediff, but introduced
+ * to allow minimal changes wrt. the removed intervals required for
+ * the ICPC specification.
+ */
+function calcContestTime($walltime)
+{
+	global $cdata;
+
+	$contesttime = difftime($walltime, $cdata['starttime']);
+
+	return $contesttime;
+}
+
 /**
  * Scoreboard calculation
  *
@@ -92,43 +105,65 @@ function problemVisible($probid)
 function calcScoreRow($cid, $team, $prob) {
 	global $DB;
 
-	$result = $DB->q('SELECT result, verified,
-	                  (UNIX_TIMESTAMP(submittime)-UNIX_TIMESTAMP(c.starttime))/60 AS timediff,
+	// First acquire an advisory lock to prevent other calls to
+	// calcScoreRow() from interfering with our update.
+	$lockstr = "domjudge.$cid.$team.$prob";
+	if ( $DB->q("VALUE SELECT GET_LOCK('$lockstr',3)") != 1 ) {
+		error("calcScoreRow failed to obtain lock '$lockstr'");
+	}
+
+	// Note the clause 'submittime < c.endtime': this is used to
+	// filter out TOO-LATE submissions from pending, but it also means
+	// that these will not count as solved. Correct submissions with
+	// submittime after contest end should never happen, unless one
+	// resets the contest time after successful judging.
+	$result = $DB->q('SELECT result, verified, submittime,
 	                  (c.freezetime IS NOT NULL && submittime >= c.freezetime) AS afterfreeze
-	                  FROM judging j
-	                  LEFT JOIN submission s USING(submitid)
+	                  FROM submission s
+	                  LEFT JOIN judging j ON(s.submitid=j.submitid AND j.valid=1)
 	                  LEFT OUTER JOIN contest c ON(c.cid=s.cid)
-	                  WHERE teamid = %s AND probid = %s AND j.valid = 1 AND
-	                  result IS NOT NULL AND s.cid = %i AND s.valid = 1
-					  ORDER BY submittime',
+	                  WHERE teamid = %s AND probid = %s AND s.cid = %i AND s.valid = 1
+	                  AND submittime < c.endtime
+	                  ORDER BY submittime',
 	                 $team, $prob, $cid);
 
-	$balloon = $DB->q('MAYBEVALUE SELECT balloon FROM scoreboard_jury
-                       WHERE cid = %i AND teamid = %s AND probid = %s',
-	                  $cid, $team, $prob);
-
-	if ( ! $balloon ) $balloon = 0;
-
 	// reset vars
-	$submitted_j = $time_j = $correct_j = 0;
-	$submitted_p = $time_p = $correct_p = 0;
+	$submitted_j = $pending_j = $time_j = $correct_j = 0;
+	$submitted_p = $pending_p = $time_p = $correct_p = 0;
 
 	// for each submission
 	while( $row = $result->next() ) {
 
-		if ( VERIFICATION_REQUIRED && ! $row['verified'] ) continue;
+		// Contest submit time in minutes for scoring.
+		$submittime = (int)floor(calcContestTime($row['submittime']) / 60);
+
+		// Check if this submission has a publicly visible judging result:
+		if ( (dbconfig_get('verification_required', 0) && ! $row['verified']) ||
+		     empty($row['result']) ) {
+
+			$pending_j++;
+			$pending_p++;
+			// Don't do any more counting for this submission.
+			continue;
+		}
 
 		$submitted_j++;
-		if ( ! $row['afterfreeze'] ) $submitted_p++;
+		if ( $row['afterfreeze'] ) {
+			// Show submissions after freeze as pending to the public
+			// (if SHOW_PENDING is enabled):
+			$pending_p++;
+		} else {
+			$submitted_p++;
+		}
 
 		// if correct, don't look at any more submissions after this one
 		if ( $row['result'] == 'correct' ) {
 
 			$correct_j = 1;
-			$time_j = round((int)@$row['timediff']);
+			$time_j = $submittime;
 			if ( ! $row['afterfreeze'] ) {
 				$correct_p = 1;
-				$time_p = round((int)@$row['timediff']);
+				$time_p = $submittime;
 			}
 			// stop counting after a first correct submission
 			break;
@@ -137,15 +172,19 @@ function calcScoreRow($cid, $team, $prob) {
 
 	// insert or update the values in the public/team scores table
 	$DB->q('REPLACE INTO scoreboard_public
-	        (cid, teamid, probid, submissions, totaltime, is_correct)
-	        VALUES (%i,%s,%s,%i,%i,%i)',
-	       $cid, $team, $prob, $submitted_p, $time_p, $correct_p);
+	        (cid, teamid, probid, submissions, pending, totaltime, is_correct)
+	        VALUES (%i,%s,%s,%i,%i,%i,%i)',
+	       $cid, $team, $prob, $submitted_p, $pending_p, $time_p, $correct_p);
 
 	// insert or update the values in the jury scores table
 	$DB->q('REPLACE INTO scoreboard_jury
-	        (cid, teamid, probid, submissions, totaltime, is_correct, balloon)
+	        (cid, teamid, probid, submissions, pending, totaltime, is_correct)
 	        VALUES (%i,%s,%s,%i,%i,%i,%i)',
-	       $cid, $team, $prob, $submitted_j, $time_j, $correct_j, $balloon);
+	       $cid, $team, $prob, $submitted_j, $pending_j, $time_j, $correct_j);
+
+	if ( $DB->q("VALUE SELECT RELEASE_LOCK('$lockstr')") != 1 ) {
+		error("calcScoreRow failed to release lock '$lockstr'");
+	}
 
 	return;
 }
@@ -159,7 +198,8 @@ function calcScoreRow($cid, $team, $prob) {
  */
 function getFinalResult($runresults)
 {
-	global $RESULTS_PRIO, $RESULTS_REMAP;
+	$results_prio  = dbconfig_get('results_prio');
+	$lazy_eval     = dbconfig_get('lazy_eval_results', true);
 
 	// Whether we have NULL results
 	$havenull = FALSE;
@@ -173,7 +213,7 @@ function getFinalResult($runresults)
 		if ( $res===NULL ) {
 			$havenull = TRUE;
 		} else {
-			$prio = $RESULTS_PRIO[$res];
+			$prio = $results_prio[$res];
 			if ( empty($prio) ) error("Unknown result '$res' found.");
 			if ( $prio>$bestprio ) {
 				$bestres  = $res;
@@ -183,13 +223,13 @@ function getFinalResult($runresults)
 	}
 
 	// Not all results are in yet, and we don't do lazy evaluation:
-	if ( $havenull && ! LAZY_EVAL_RESULTS ) return NULL;
+	if ( $havenull && ! $lazy_eval ) return NULL;
 
-	if ( LAZY_EVAL_RESULTS ) {
+	if ( $lazy_eval ) {
 		// If we have NULL results, check whether the highest priority
 		// result has maximal priority. Use a local copy of the
-		// RESULTS_PRIO array, keeping the original untouched.
-		$tmp = $RESULTS_PRIO;
+		// 'results_prio' array, keeping the original untouched.
+		$tmp = $results_prio;
 		rsort($tmp);
 		$maxprio = reset($tmp);
 
@@ -197,12 +237,7 @@ function getFinalResult($runresults)
 		if ( $havenull && $bestprio<$maxprio ) return NULL;
 	}
 
-	// We have a (possibly lazy) final answer, check for remapping.
-	if ( array_key_exists($bestres, $RESULTS_REMAP) ) {
-		return $RESULTS_REMAP[$bestres];
-	} else {
-		return $bestres;
-	}
+	return $bestres;
 }
 
 /**
@@ -242,13 +277,13 @@ function now()
 
 /**
  * Returns >0, =0, <0 when $time1 >, =, < $time2 respectively.
- * This function currently uses string-based compare on the MySQL
- * format (see above), but is abstracted here for possible changes,
- * e.g. a C-like implementation with a numeric representation.
+ * This function converts the strings to integer seconds and returns
+ * their difference. We don't use the default second argument 'now()'
+ * for 'strtotime()' since it could (theoretically) change.
  */
 function difftime($time1, $time2)
 {
-	return strcmp($time1, $time2);
+	return strtotime($time1, 0) - strtotime($time2, 0);
 }
 
 /**
@@ -487,28 +522,40 @@ function daemonize($pidfile = NULL)
 	logmsg(LOG_NOTICE, "daemonizing with PID = $pid");
 
 	// Close std{in,out,err} file descriptors
-	if ( !fclose(STDIN ) ||
-	     !fclose(STDOUT) ||
-	     !fclose(STDERR) ) error("cannot close stdio files");
+	if ( !fclose(STDIN ) || !($GLOBALS['STDIN']  = fopen('/dev/null', 'r')) ||
+	     !fclose(STDOUT) || !($GLOBALS['STDOUT'] = fopen('/dev/null', 'w')) ||
+	     !fclose(STDERR) || !($GLOBALS['STDERR'] = fopen('/dev/null', 'w')) ) {
+		error("cannot reopen stdio files to /dev/null");
+	}
 
 	// Start own process group, detached from any tty
 	if ( posix_setsid()<0 ) error("cannot set daemon process group");
 }
 
 /**
- * This function takes a temporary file of a submission,
+ * This function takes a (set of) temporary file(s) of a submission,
  * validates it and puts it into the database. Additionally it
  * moves it to a backup storage.
  */
-function submit_solution($team, $prob, $lang, $file)
+function submit_solution($team, $prob, $lang, $files, $filenames, $origsubmitid = NULL)
 {
 	if( empty($team) ) error("No value for Team.");
 	if( empty($prob) ) error("No value for Problem.");
 	if( empty($lang) ) error("No value for Language.");
-	if( empty($file) ) error("No value for Filename.");
+
+	if ( !is_array($files) || count($files)==0 ) error("No files specified.");
+	if ( !is_array($filenames) || count($filenames)!=count($files) ) {
+		error("Nonmatching (number of) filenames specified.");
+	}
+
+	if ( count($filenames)!=count(array_unique($filenames)) ) {
+		error("Duplicate filenames detected.");
+	}
 
 	global $cdata,$cid, $DB;
 
+	$sourcesize = dbconfig_get('sourcesize_limit');
+
 	// If no contest has started yet, refuse submissions.
 	$now = now();
 
@@ -529,34 +576,62 @@ function submit_solution($team, $prob, $lang, $file)
 							AND cid = %i AND allow_submit = "1"', $prob, $cid) ) {
 		error("Problem '$prob' not found in database or not submittable [c$cid].");
 	}
-	if( ! is_readable($file) ) {
-		error("File '$file' not found (or not readable).");
+
+	// Reindex arrays numerically to allow simultaneously iterating
+	// over both $files and $filenames.
+	$files     = array_values($files);
+	$filenames = array_values($filenames);
+
+	$totalsize = 0;
+	for($i=0; $i<count($files); $i++) {
+		if ( ! is_readable($files[$i]) ) {
+			error("File '".$files[$i]."' not found (or not readable).");
+		}
+		if ( ! preg_match(FILENAME_REGEX, $filenames[$i]) ) {
+			error("Illegal filename '".$filenames[$i]."'.");
+		}
+		$totalsize += filesize($files[$i]);
 	}
-	if( filesize($file) > SOURCESIZE*1024 ) {
-		error("Submission file is larger than ".SOURCESIZE." kB.");
+	if ( $totalsize > $sourcesize*1024 ) {
+		error("Submission file(s) are larger than $sourcesize kB.");
 	}
 
 	logmsg (LOG_INFO, "input verified");
 
 	// Insert submission into the database
 	$id = $DB->q('RETURNID INSERT INTO submission
-				  (cid, teamid, probid, langid, submittime, sourcecode)
-				  VALUES (%i, %s, %s, %s, %s, %s)',
-				 $cid, $team, $probid, $langid, $now,
-				 getFileContents($file, false));
+				  (cid, teamid, probid, langid, submittime, origsubmitid)
+				  VALUES (%i, %s, %s, %s, %s, %i)',
+	             $cid, $team, $probid, $langid, $now, $origsubmitid);
+
+	for($rank=0; $rank<count($files); $rank++) {
+		$DB->q('INSERT INTO submission_file
+		        (submitid, filename, rank, sourcecode) VALUES (%i, %s, %i, %s)',
+		       $id, $filenames[$rank], $rank, getFileContents($files[$rank], false));
+	}
+
+	// Recalculate scoreboard cache for pending submissions
+	calcScoreRow($cid, $team, $probid);
 
 	// Log to event table
 	$DB->q('INSERT INTO event (eventtime, cid, teamid, langid, probid, submitid, description)
-			VALUES(%s, %i, %s, %s, %s, %i, "problem submitted")',
-		   now(), $cid, $team, $langid, $probid, $id);
-
-	$tofile = getSourceFilename($cid,$id,$team,$probid,$langid);
-	$topath = SUBMITDIR . "/$tofile";
+	        VALUES(%s, %i, %s, %s, %s, %i, "problem submitted")',
+	       now(), $cid, $team, $langid, $probid, $id);
 
 	if ( is_writable( SUBMITDIR ) ) {
 		// Copy the submission to SUBMITDIR for safe-keeping
-		if ( ! @copy($file, $topath) ) {
-			warning("Could not copy '" . $file . "' to '" . $topath . "'");
+		for($rank=0; $rank<count($files); $rank++) {
+			$fdata = array('cid' => $cid,
+			               'submitid' => $id,
+			               'teamid' => $team,
+			               'probid' => $probid,
+			               'langid' => $langid,
+			               'rank' => $rank,
+			               'filename' => $filenames[$rank]);
+			$tofile = SUBMITDIR . '/' . getSourceFilename($fdata);
+			if ( ! @copy($files[$rank], $tofile) ) {
+				warning("Could not copy '" . $files[$rank] . "' to '" . $tofile . "'");
+			}
 		}
 	} else {
 		logmsg(LOG_DEBUG, "SUBMITDIR not writable, skipping");
@@ -570,11 +645,14 @@ function submit_solution($team, $prob, $lang, $file)
 }
 
 /**
- * Compute the filename of a given submission.
+ * Compute the filename of a given submission. $fdata must be an array
+ * that contains the data from submission and submission_file.
  */
-function getSourceFilename($cid,$sid,$team,$prob,$lang)
+function getSourceFilename($fdata)
 {
-	return "c$cid.s$sid.$team.$prob.$lang";
+	return implode('.', array('c'.$fdata['cid'], 's'.$fdata['submitid'],
+	                          $fdata['teamid'], $fdata['probid'], $fdata['langid'],
+	                          $fdata['rank'], $fdata['filename']));
 }
 
 /**
@@ -673,3 +751,24 @@ function XMLgetattr($node, $attr)
 
 	return $attrnode->nodeValue;
 }
+
+/**
+ * Log an action to the auditlog table.
+ */
+function auditlog($datatype, $dataid, $action, $extrainfo = null, $username = null)
+{
+	global $cid, $login, $DB;
+
+	if ( !empty($username) ) {
+		$user = $username;
+	} elseif ( IS_JURY ) {
+		$user = getJuryMember();
+	} else {
+		$user = $login;
+	}
+
+	$DB->q('INSERT INTO auditlog
+	        (logtime, cid, user, datatype, dataid, action, extrainfo)
+	        VALUES(%s, %i, %s, %s, %s, %s, %s)',
+	       now(), $cid, $user, $datatype, $dataid, $action, $extrainfo);
+}
