@@ -1,9 +1,8 @@
 <?php
-/* $Id: use_db.php 3500 2010-12-15 20:50:23Z kink $ */
 
 require('lib.database.php');
 
-function setup_database_connection($privlevel)
+function setup_database_connection()
 {
 	$credfile = ETCDIR . '/dbpasswords.secret';
 	$credentials = @file($credfile);
@@ -25,15 +24,17 @@ function setup_database_connection($privlevel)
 		if ( $credential{0} == '#' ) continue;
 		list ($priv, $host, $db, $user, $pass) =
 			explode(':', trim($credential));
-		if ($priv != $privlevel) continue;
+		if ( $priv=='team' || $priv=='public' ) {
+			user_error("Found obsolete database privilege:user '$priv:$user'", E_USER_WARNING);
+			continue;
+		}
 
 		$DB = new db ($db, $host, $user, $pass);
 		break;
 	}
 
 	if (!$DB) {
-		user_error(sprintf("Privilege level '%s' not supported",
-			$privlevel), E_USER_ERROR);
+		user_error("Failed to create database connection", E_USER_ERROR);
 		exit();
 	}
 
