@@ -6,8 +6,6 @@
  * to the team ID and $teamdata contains the corresponding row from
  * the database. $ip is set to the remote IP address used.
  *
- * $Id: auth.team.php 3536 2011-02-10 12:38:00Z werth $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -167,7 +165,7 @@ function ldap_check_credentials($user, $pass)
 		$ldap_dn = str_replace('&', $user, LDAP_DNQUERY);
 
 		// Try to login to test credentials
-		if ( @ldap_bind($conn, $ldap_dn, $passwd) ) {
+		if ( @ldap_bind($conn, $ldap_dn, $pass) ) {
 			@ldap_unbind($conn);
 			return TRUE;
 		}
@@ -217,6 +215,7 @@ function do_login()
 		if ( AUTH_METHOD=='PHP_SESSIONS' ) {
 			session_start();
 			$_SESSION['teamid'] = $login;
+			auditlog('team', $login, 'logged in', $_SERVER['REMOTE_ADDR']);
 		}
 		break;
 
@@ -245,6 +244,7 @@ function do_login()
 
 		session_start();
 		$_SESSION['teamid'] = $login;
+		auditlog('team', $login, 'logged in', $_SERVER['REMOTE_ADDR']);
 		break;
 
 	default:
@@ -289,6 +289,7 @@ function do_logout()
 
 	$title = 'Logout';
 	$menu = FALSE;
+	auditlog('team', $login, 'logged out', $_SERVER['REMOTE_ADDR']);
 
 	require(LIBWWWDIR . '/header.php');
 	echo "<h1>Logged out</h1>\n\n<p>Successfully logged out as team " .
