@@ -2,8 +2,6 @@
 /**
  * Clarification helper functions for jury and teams
  *
- * $Id: clarification.php 3487 2010-12-05 12:17:52Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -40,17 +38,19 @@ function canViewClarification($team, $clar)
  */
 function putClar($clar)
 {
-	if ( $clar['sender'] ) {
+	// $clar['sender'] is set to the team ID, or empty if sent by the jury.
+	if ( !empty($clar['sender']) ) {
 		$from = '<span class="teamid">' . htmlspecialchars($clar['sender']) .
 			'</span>: ' . htmlspecialchars($clar['fromname']);
 	} else {
 		$from = 'Jury';
+		if ( IS_JURY ) $from .= ' (' . htmlspecialchars($clar['jury_member']) . ')';
 	}
-	if ( $clar['recipient'] && $from == 'Jury' ) {
+	if ( $clar['recipient'] && empty($clar['sender']) ) {
 		$to = '<span class="teamid">' . htmlspecialchars($clar['recipient']) .
 			'</span>: ' . htmlspecialchars($clar['toname']);
 	} else {
-		$to = ( $from == 'Jury' ) ? 'All' : 'Jury' ;
+		$to = ( $clar['sender'] ) ? 'Jury' : 'All';
 	}
 
 	echo "<table>\n";
@@ -137,11 +137,12 @@ function putClarification($id,  $team = NULL)
  */
 function summarizeClarification($body)
 {
-	// when making a summary, try to igonore the quoted text
+	// when making a summary, try to ignore the quoted text, and
+	// replace newlines by spaces.
 	$split = explode("\n", $body);
 	$newbody = '';
 	foreach($split as $line) {
-		if ( strlen($line) > 0 && $line{0} != '>' ) $newbody .= $line;
+		if ( strlen($line) > 0 && $line{0} != '>' ) $newbody .= $line . ' ';
 	}
 	return htmlspecialchars( str_cut( ( empty($newbody) ? $body : $newbody ), 80) );
 }
@@ -155,8 +156,7 @@ function putClarificationList($clars, $team = NULL)
 		error("access denied to clarifications: you seem to be team nor jury");
 	}
 
-	echo "<table class=\"list sortable\">\n<thead>\n";
-	echo "<tr>" .
+	echo "<table class=\"list sortable\">\n<thead>\n<tr>" .
 		( IS_JURY ? "<th scope=\"col\">ID</th>" : "") .
 	     "<th scope=\"col\">time</th>" .
 	     "<th scope=\"col\">from</th>" .
@@ -167,22 +167,22 @@ function putClarificationList($clars, $team = NULL)
 
 	while ( $clar = $clars->next() ) {
 		// check viewing permission for teams
-		if ( ! IS_JURY && !canViewClarification($team, $clar))
-			continue;
+		if ( ! IS_JURY && !canViewClarification($team, $clar) ) continue;
 
 		$clar['clarid'] = (int)$clar['clarid'];
 		$link = '<a href="clarification.php?id=' . urlencode($clar['clarid'])  . '">';
 
-		if(isset($clar['unread']))
+		if ( isset($clar['unread']) ) {
 			echo '<tr class="unread">';
-		else
+		} else {
 			echo '<tr>';
+		}
 
-		if ( IS_JURY ) 
+		if ( IS_JURY ) {
 			echo '<td>' . $link . $clar['clarid'] . '</a></td>';
+		}
 
-		echo '<td>' . $link;
-		echo printtime($clar['submittime']) . '</a></td>';
+		echo '<td>' . $link . printtime($clar['submittime']) . '</a></td>';
 
 		$sender = htmlspecialchars($clar['sender']);
 		$recipient = htmlspecialchars($clar['recipient']);
@@ -191,18 +191,12 @@ function putClarificationList($clars, $team = NULL)
 			$sender = 'Jury';
 			$recipient = 'All';
 		} else {
-			if ($sender == NULL)
-				$sender = 'Jury';
-
-			if ($recipient == NULL)
-				$recipient = 'Jury';
+			if ( $sender    == NULL ) $sender = 'Jury';
+			if ( $recipient == NULL ) $recipient = 'Jury';
 		}
 
-
-		echo '<td class="teamid">' . $link .
-			 $sender . '</a></td>';
-		echo '<td class="teamid">' . $link .
-			 $recipient . '</a></td>';
+		echo '<td class="teamid">' . $link . $sender . '</a></td>' .
+		     '<td class="teamid">' . $link . $recipient . '</a></td>';
 
 		echo '<td>' . $link;
 		if ( is_null($clar['probid']) ) {
@@ -212,9 +206,9 @@ function putClarificationList($clars, $team = NULL)
 		}
 		echo "</a></td>";
 
-		echo '<td class="clartext">' . $link;
-		echo summarizeClarification($clar['body']);
-		echo "</a></td>";
+		echo '<td class="clartext">' . $link .
+		    summarizeClarification($clar['body']) . "</a></td>";
+
 		if ( IS_JURY ) {
 			unset($answered, $jury_member);
 			$claim = FALSE;
@@ -234,10 +228,12 @@ function putClarificationList($clars, $team = NULL)
 
 			echo "<td>$link $answered</a></td><td>";
 			if ( $claim && isset($clar['sender']) ) {
-				echo "<a class=\"button\" href=\"clarification.php?claim=1&id=" . htmlspecialchars($clar['clarid']) . "\">claim</a>";
+				echo "<a class=\"button\" href=\"clarification.php?claim=1&amp;id=" .
+					htmlspecialchars($clar['clarid']) . "\">claim</a>";
 			} else {
 				if ( !$clar['answered'] && $jury_member==getJuryMember() ) {
-					echo "<a class=\"button\" href=\"clarification.php?unclaim=1&id=" . htmlspecialchars($clar['clarid']) . "\">unclaim</a>";
+					echo "<a class=\"button\" href=\"clarification.php?unclaim=1&amp;id=" .
+						htmlspecialchars($clar['clarid']) . "\">unclaim</a>";
 				} else {
 					echo "$link $jury_member</a>";
 				}
@@ -255,6 +251,11 @@ function putClarificationList($clars, $team = NULL)
  */
 function putClarificationForm($action, $cid, $respid = NULL)
 {
+	if ( empty($cid) ) {
+		echo '<p class="nodata">No active contest</p>';
+		return;
+	}
+
 	require_once('forms.php');
 
 	global $DB, $cdata;
@@ -301,7 +302,7 @@ function confirmClar() {
 		if ( ! $respid ) {
 			$teams = $DB->q('KEYVALUETABLE SELECT login, CONCAT(login, ": ", name) as name
 			                 FROM team
-			                 ORDER BY categoryid ASC, name ASC');
+			                 ORDER BY categoryid ASC, team.name COLLATE utf8_general_ci ASC');
 			$options = array_merge($options,$teams);
 		} else {
 			if ( $clar['sender'] ) {
@@ -352,6 +353,7 @@ echo addTextArea('bodytext', $body, 80, 10);
 <script type="text/javascript">
 <!--
 document.forms['sendclar'].bodytext.focus();
+document.forms['sendclar'].bodytext.select();
 // -->
 </script>
 <?php
