@@ -3,11 +3,35 @@
 -- You can pipe this file into the 'mysql' command to create the
 -- database tables, but preferably use 'dj-setup-database'. Database
 -- should be set externally (e.g. to 'domjudge').
---
--- $Id: mysql_db_structure.sql 3594 2011-08-13 18:59:21Z eldering $
 
 /*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
 
+--
+-- Table structure for table `auditlog`
+--
+CREATE TABLE `auditlog` (
+  `logid` int(10) unsigned NOT NULL AUTO_INCREMENT COMMENT 'Unique ID',
+  `logtime` datetime NOT NULL COMMENT 'Timestamp of the logentry',
+  `cid` int(4) unsigned DEFAULT NULL COMMENT 'Contest ID associated to this entry',
+  `user` varchar(255) DEFAULT NULL COMMENT 'User who performed this action',
+  `datatype` varchar(25) DEFAULT NULL COMMENT 'Reference to DB table associated to this entry',
+  `dataid` varchar(50) DEFAULT NULL COMMENT 'Identifier in reference table',
+  `action` varchar(30) DEFAULT NULL COMMENT 'Description of action performed',
+  `extrainfo` varchar(255) DEFAULT NULL COMMENT 'Optional additional description of the entry',
+  PRIMARY KEY (`logid`)
+) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Log of all actions performed';
+
+--
+-- Table structure for table `balloon`
+--
+CREATE TABLE `balloon` (
+  `balloonid` int(4) unsigned NOT NULL AUTO_INCREMENT COMMENT 'Unique ID',
+  `submitid` int(4) unsigned NOT NULL COMMENT 'Submission for which balloon was earned',
+  `done` tinyint(1) unsigned NOT NULL DEFAULT '0' COMMENT 'Has been handed out yet?',
+  PRIMARY KEY (`balloonid`),
+  KEY `submitid` (`submitid`),
+  CONSTRAINT `balloon_ibfk_1` FOREIGN KEY (`submitid`) REFERENCES `submission` (`submitid`) ON DELETE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Balloons to be handed out';
 
 --
 -- Table structure for table `clarification`
@@ -21,7 +45,7 @@ CREATE TABLE `clarification` (
   `recipient` varchar(15) default NULL COMMENT 'Team login, null means to jury or to all',
   `jury_member` varchar(15) default NULL COMMENT 'Name of jury member who answered this',
   `probid` varchar(8) default NULL COMMENT 'Problem associated to this clarification',
-  `body` text NOT NULL COMMENT 'Clarification text',
+  `body` longtext NOT NULL COMMENT 'Clarification text',
   `answered` tinyint(1) unsigned NOT NULL default '0' COMMENT 'Has been answered by jury?',
   PRIMARY KEY  (`clarid`),
   KEY `cid` (`cid`,`answered`,`submittime`),
@@ -37,9 +61,13 @@ CREATE TABLE `clarification` (
 --
 
 CREATE TABLE `configuration` (
+  `configid` int(4) NOT NULL AUTO_INCREMENT COMMENT 'Configuration ID',
   `name` varchar(25) NOT NULL COMMENT 'Name of the configuration variable',
-  `value` varchar(255) NOT NULL COMMENT 'Content of the configuration variable',
-  PRIMARY KEY  (`name`)
+  `value` longtext NOT NULL COMMENT 'Content of the configuration variable',
+  `type` varchar(25) default NULL COMMENT 'Type of the value (metatype for use in the webinterface)',
+  `description` varchar(255) default NULL COMMENT 'Description for in the webinterface',
+  PRIMARY KEY (`configid`),
+  KEY `name` (`name`)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Global configuration variables';
 
 
@@ -55,10 +83,10 @@ CREATE TABLE `contest` (
   `freezetime` datetime default NULL COMMENT 'Time scoreboard is frozen',
   `endtime` datetime NOT NULL COMMENT 'Time after which no more submissions are accepted',
   `unfreezetime` datetime default NULL COMMENT 'Unfreeze a frozen scoreboard at this time',
-  `activatetime_string` varchar(20) NOT NULL COMMENT 'Time contest becomes visible in team/public views',
-  `freezetime_string` varchar(20) default NULL COMMENT 'Time scoreboard is frozen',
-  `endtime_string` varchar(20) NOT NULL COMMENT 'Time after which no more submissions are accepted',
-  `unfreezetime_string` varchar(20) default NULL COMMENT 'Unfreeze a frozen scoreboard at this time',
+  `activatetime_string` varchar(20) NOT NULL COMMENT 'Authoritative absolute or relative string representation of activatetime',
+  `freezetime_string` varchar(20) default NULL COMMENT 'Authoritative absolute or relative string representation of freezetime',
+  `endtime_string` varchar(20) NOT NULL COMMENT 'Authoritative absolute or relative string representation of endtime',
+  `unfreezetime_string` varchar(20) default NULL COMMENT 'Authoritative absolute or relative string representation of unfreezetrime',
   `enabled` tinyint(1) unsigned NOT NULL default '1' COMMENT 'Whether this contest can be active',
   PRIMARY KEY (`cid`),
   KEY `cid` (`cid`,`enabled`)
@@ -78,7 +106,7 @@ CREATE TABLE `event` (
   `submitid` int(4) unsigned default NULL COMMENT 'Submission ID',
   `judgingid` int(4) unsigned default NULL COMMENT 'Judging ID',
   `teamid` varchar(15) default NULL COMMENT 'Team login',
-  `description` text NOT NULL COMMENT 'Event description',
+  `description` longtext NOT NULL COMMENT 'Event description',
   PRIMARY KEY  (`eventid`)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Log of all events during a contest';
 
@@ -107,8 +135,10 @@ CREATE TABLE `judging` (
   `result` varchar(25) default NULL COMMENT 'Result string as defined in config.php',
   `verified` tinyint(1) unsigned NOT NULL default '0' COMMENT 'Result verified by jury member?',
   `jury_member` varchar(15) default NULL COMMENT 'Name of jury member who verified this',
+  `verify_comment` varchar(255) DEFAULT NULL COMMENT 'Optional additional information provided by the verifier',
   `valid` tinyint(1) unsigned NOT NULL default '1' COMMENT 'Old judging is marked as invalid when rejudging',
-  `output_compile` text COMMENT 'Output of the compiling the program',
+  `output_compile` longblob COMMENT 'Output of the compiling the program',
+  `seen` tinyint(1) unsigned NOT NULL default '0' COMMENT 'Whether the team has seen this judging',
   PRIMARY KEY  (`judgingid`),
   KEY `submitid` (`submitid`),
   KEY `judgehost` (`judgehost`),
@@ -128,9 +158,9 @@ CREATE TABLE `judging_run` (
   `testcaseid` int(4) unsigned NOT NULL COMMENT 'Testcase ID',
   `runresult` varchar(25) default NULL COMMENT 'Result of this run, NULL if not finished yet',
   `runtime` float DEFAULT NULL COMMENT 'Submission running time on this testcase',
-  `output_run` text COMMENT 'Output of running the program',
-  `output_diff` text COMMENT 'Diffing the program output and testcase output',
-  `output_error` text COMMENT 'Standard error output of the program',
+  `output_run` longblob COMMENT 'Output of running the program',
+  `output_diff` longblob COMMENT 'Diffing the program output and testcase output',
+  `output_error` longblob COMMENT 'Standard error output of the program',
   PRIMARY KEY  (`runid`),
   UNIQUE KEY `testcaseid` (`judgingid`, `testcaseid`),
   KEY `judgingid` (`judgingid`),
@@ -180,9 +210,9 @@ CREATE TABLE `scoreboard_jury` (
   `teamid` varchar(15) NOT NULL COMMENT 'Team login',
   `probid` varchar(8) NOT NULL COMMENT 'Problem ID',
   `submissions` int(4) unsigned NOT NULL default '0' COMMENT 'Number of submissions made',
+  `pending` int(4) NOT NULL default '0' COMMENT 'Number of submissions pending judgement',
   `totaltime` int(4) unsigned NOT NULL default '0' COMMENT 'Total time spent',
   `is_correct` tinyint(1) unsigned NOT NULL default '0' COMMENT 'Has there been a correct submission?',
-  `balloon` tinyint(1) unsigned NOT NULL default '0' COMMENT 'Has a balloon been handed out?',
   PRIMARY KEY  (`cid`,`teamid`,`probid`)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Scoreboard cache (jury version)';
 
@@ -195,6 +225,7 @@ CREATE TABLE `scoreboard_public` (
   `teamid` varchar(15) NOT NULL COMMENT 'Team login',
   `probid` varchar(8) NOT NULL COMMENT 'Problem ID',
   `submissions` int(4) unsigned NOT NULL default '0' COMMENT 'Number of submissions made',
+  `pending` int(4) NOT NULL default '0' COMMENT 'Number of submissions pending judgement',
   `totaltime` int(4) unsigned NOT NULL default '0' COMMENT 'Total time spent',
   `is_correct` tinyint(1) unsigned NOT NULL default '0' COMMENT 'Has there been a correct submission?',
   PRIMARY KEY  (`cid`,`teamid`,`probid`)
@@ -206,12 +237,12 @@ CREATE TABLE `scoreboard_public` (
 
 CREATE TABLE `submission` (
   `submitid` int(4) unsigned NOT NULL auto_increment COMMENT 'Unique ID',
+  `origsubmitid` int(4) unsigned default NULL COMMENT 'If set, specifies original submission in case of edit/resubmit',
   `cid` int(4) unsigned NOT NULL COMMENT 'Contest ID',
   `teamid` varchar(15) NOT NULL COMMENT 'Team login',
   `probid` varchar(8) NOT NULL COMMENT 'Problem ID',
   `langid` varchar(8) NOT NULL COMMENT 'Language ID',
   `submittime` datetime NOT NULL COMMENT 'Time submitted',
-  `sourcecode` mediumblob NOT NULL COMMENT 'Full source code',
   `judgehost` varchar(50) default NULL COMMENT 'Current/last judgehost judging this submission',
   `judgemark` varchar(255) default NULL COMMENT 'Unique identifier for taking a submission by a judgehost',
   `valid` tinyint(1) unsigned NOT NULL default '1' COMMENT 'If false ignore this submission in all scoreboard calculations',
@@ -223,26 +254,47 @@ CREATE TABLE `submission` (
   KEY `probid` (`probid`),
   KEY `langid` (`langid`),
   KEY `judgehost_2` (`judgehost`),
+  KEY `origsubmitid` (`origsubmitid`),
   CONSTRAINT `submission_ibfk_1` FOREIGN KEY (`cid`) REFERENCES `contest` (`cid`) ON DELETE CASCADE,
   CONSTRAINT `submission_ibfk_2` FOREIGN KEY (`teamid`) REFERENCES `team` (`login`) ON DELETE CASCADE,
   CONSTRAINT `submission_ibfk_3` FOREIGN KEY (`probid`) REFERENCES `problem` (`probid`) ON DELETE CASCADE,
   CONSTRAINT `submission_ibfk_4` FOREIGN KEY (`langid`) REFERENCES `language` (`langid`) ON DELETE CASCADE,
-  CONSTRAINT `submission_ibfk_5` FOREIGN KEY (`judgehost`) REFERENCES `judgehost` (`hostname`) ON DELETE SET NULL
+  CONSTRAINT `submission_ibfk_5` FOREIGN KEY (`judgehost`) REFERENCES `judgehost` (`hostname`) ON DELETE SET NULL,
+  CONSTRAINT `submission_ibfk_6` FOREIGN KEY (`origsubmitid`) REFERENCES `submission` (`submitid`) ON DELETE SET NULL
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='All incoming submissions';
 
+--
+-- Table structure for table `submission_file`
+--
+
+CREATE TABLE `submission_file` (
+  `submitfileid` int(4) unsigned NOT NULL AUTO_INCREMENT COMMENT 'Unique ID',
+  `submitid` int(4) unsigned NOT NULL COMMENT 'Submission this file belongs to',
+  `sourcecode` longblob NOT NULL COMMENT 'Full source code',
+  `filename` varchar(255) NOT NULL COMMENT 'Filename as submitted',
+  `rank` int(4) unsigned NOT NULL COMMENT 'Order of the submission files, zero-indexed',
+  PRIMARY KEY (`submitfileid`),
+  UNIQUE KEY `filename` (`submitid`,`filename`),
+  UNIQUE KEY `rank` (`submitid`,`rank`),
+  KEY `submitid` (`submitid`),
+  CONSTRAINT `submission_file_ibfk_1` FOREIGN KEY (`submitid`) REFERENCES `submission` (`submitid`) ON DELETE CASCADE
+) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Files associated to a submission';
+
 --
 -- Table structure for table `team`
 --
 
 CREATE TABLE `team` (
   `login` varchar(15) NOT NULL COMMENT 'Team login name',
-  `name` varchar(255) NOT NULL COMMENT 'Team name',
+  `name` varchar(255) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL COMMENT 'Team name',
   `categoryid` int(4) unsigned NOT NULL default '0' COMMENT 'Team category ID',
   `affilid` varchar(10) default NULL COMMENT 'Team affiliation ID',
   `authtoken` varchar(255) default NULL COMMENT 'Identifying token for this team',
-  `members` text COMMENT 'Team member names (freeform)',
+  `enabled` tinyint(1) unsigned NOT NULL DEFAULT '1' COMMENT 'Whether the team is visible and operational',
+  `members` longtext COMMENT 'Team member names (freeform)',
   `room` varchar(15) default NULL COMMENT 'Physical location of team',
-  `comments` text COMMENT 'Comments about this team',
+  `comments` longtext COMMENT 'Comments about this team',
+  `judging_last_started` datetime default NULL COMMENT 'Start time of last judging for priorization',
   `teampage_first_visited` datetime default NULL COMMENT 'Time of first teampage view',
   `hostname` varchar(255) default NULL COMMENT 'Teampage first visited from this address',
   PRIMARY KEY  (`login`),
@@ -260,8 +312,8 @@ CREATE TABLE `team` (
 CREATE TABLE `team_affiliation` (
   `affilid` varchar(10) NOT NULL COMMENT 'Unique ID',
   `name` varchar(255) NOT NULL COMMENT 'Descriptive name',
-  `country` char(2) default NULL COMMENT 'ISO country code',
-  `comments` text COMMENT 'Comments',
+  `country` char(3) default NULL COMMENT 'ISO 3166-1 alpha-3 country code',
+  `comments` longtext COMMENT 'Comments',
   PRIMARY KEY  (`affilid`)
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='Affilitations for teams (e.g.: university, company)';
 
