@@ -3,8 +3,6 @@
  * Tool to coordinate the handing out of balloons to teams that solved
  * a problem. Similar to the balloons-daemon, but web-based.
  *
- * $Id: balloons.php 3494 2010-12-10 23:23:59Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -14,10 +12,10 @@ $title = 'Balloon Status';
 
 if ( isset($_POST['done']) ) {
 	foreach($_POST['done'] as $done => $dummy) {
-		$parts = explode(';', $done);
-		$DB->q('UPDATE scoreboard_jury SET balloon=1
-			WHERE probid = %s AND teamid = %s AND cid = %i',
-			$parts[0], $parts[1], $parts[2]);
+		$DB->q('UPDATE balloon SET done=1
+			WHERE balloonid = %i',
+			$done);
+		auditlog('balloon', $done, 'marked done');
 	}
 	header('Location: balloons.php');
 }
@@ -32,14 +30,7 @@ if ( isset($_COOKIE['domjudge_balloonviewall']) ) {
 // Did someone press the view button?
 if ( isset($_REQUEST['viewall']) ) $viewall = $_REQUEST['viewall'];
 
-// Set cookie of submission view type, expiry defaults to end of session.
-if ( version_compare(PHP_VERSION, '5.2') >= 0 ) {
-	// HTTPOnly Cookie, while this cookie is not security critical
-	// it's a good habit to get into.
-	setcookie('domjudge_balloonviewall', $viewall, null, null, null, null, true);
-} else {
-	setcookie('domjudge_balloonviewall', $viewall);
-}
+setcookie('domjudge_balloonviewall', $viewall);
 
 $refresh = '30;url=balloons.php';
 require(LIBWWWDIR . '/header.php');
@@ -53,22 +44,23 @@ if ( isset($cdata['freezetime']) &&
 
 echo addForm('balloons.php', 'get') . "<p>\n" .
     addHidden('viewall', ($viewall ? 0 : 1)) .
-    addSubmit($viewall ? 'view unsent' : 'view all') . "</p>\n" .
+    addSubmit($viewall ? 'view unsent only' : 'view all') . "</p>\n" .
     addEndForm();
 
 // Problem metadata: colours and names.
 $probs_data = $DB->q('KEYTABLE SELECT probid AS ARRAYKEY,name,color
 		      FROM problem WHERE cid = %i', $cid);
 
-// Get all relevant info from the scoreboard_jury table.
-// Order by balloon, so we have the unsent balloons at the top.
-// Then by submittime, so the newest will also rank highest.
-$res = $DB->q('SELECT s.*, t.login, t.name AS teamname, t.room, c.name AS catname
-               FROM scoreboard_jury s
+// Get all relevant info from the balloon table.
+// Order by done, so we have the unsent balloons at the top.
+$res = $DB->q('SELECT b.*, s.probid, s.submittime,
+               t.login, t.name AS teamname, t.room, c.name AS catname
+               FROM balloon b
+               LEFT JOIN submission s USING (submitid)
                LEFT JOIN team t ON (t.login = s.teamid)
                LEFT JOIN team_category c USING(categoryid)
-               WHERE s.cid = %i AND s.is_correct = 1
-               ORDER BY s.balloon, s.totaltime DESC',
+               WHERE s.cid = %i
+               ORDER BY done ASC, balloonid DESC',
               $cid);
 
 /* Loop over the result, store the total of balloons for a team
@@ -80,6 +72,10 @@ $BALLOONS = $TOTAL_BALLOONS = array();
 while ( $row = $res->next() ) {
 	$BALLOONS[] = $row;
 	$TOTAL_BALLOONS[$row['login']][] = $row['probid'];
+
+	// keep overwriting these variables - in the end they'll
+	// contain the id's of the first balloon in each type
+	$first_contest = $first_problem[$row['probid']] = $first_team[$row['login']] = $row['balloonid'];
 }
 
 $conteststart  = strtotime($cdata['starttime']);
@@ -91,33 +87,34 @@ if ( !empty($BALLOONS) ) {
 	echo addForm('balloons.php');
 
 	echo "<table class=\"list sortable balloons\">\n<thead>\n" .
-		"<tr><th>Time</th><th>Solved</th><th></th><th align=\"left\">Team</th>\n" .
-		"<th>Room</th><th>Category</th><th>Total</th><th></th></tr>\n</thead>\n";
+		"<tr><td></td><th class=\"sorttable_numeric\">ID</th>" .
+	        "<th>time</th><th>solved</th><th align=\"right\">team</th>" .
+	        "<th></th><th>loc.</th><th>category</th><th>total</th>" .
+	        "<th></th><th></th></tr>\n</thead>\n";
 
 	foreach ( $BALLOONS as $row ) {
 
-		if ( !$viewall && $row['balloon'] == 1 ) continue;
+		if ( !$viewall && $row['done'] == 1 ) continue;
 
 		// start a new row, 'disable' if balloon has been handed out already
-		echo '<tr'  . ( $row['balloon'] == 1 ? ' class="disabled"' : '' ) . '>';
-
-		// time the balloon was earned (contest start + total time (in minutes))
-		// display an "F" after the time if this is after the freeze.
-		$balloontime = $conteststart + ($row['totaltime']*60);
-		$frozen = (isset($contestfreeze) && $balloontime >= $contestfreeze ?
-			' <span title="After Scoreboard Freeze" class="frozen">F</span>' : '');
-
-		echo '<td>' . printtime( strftime(MYSQL_DATETIME_FORMAT, $balloontime) ) .
-			$frozen . '</td>';
+		echo '<tr'  . ( $row['done'] == 1 ? ' class="disabled"' : '' ) . '>';
+		if ( isset($cdata['freezetime']) &&
+		     $row['submittime'] > $cdata['freezetime'] ) {
+			echo "<td>FROZEN</td>";
+		} else {
+			echo '<td></td>';
+		}
+		echo '<td>b' . (int)$row['balloonid'] . '</td>';
+		echo '<td>' . printtime( $row['submittime'] ) . '</td>';
 
 		// the balloon earned
 		echo '<td class="probid">' .
-			'<img style="background-color: ' .
+			'<img class="balloonimage" style="background-color: ' .
 		    htmlspecialchars($probs_data[$row['probid']]['color']) .
 			';" alt="problem colour ' . htmlspecialchars($probs_data[$row['probid']]['color']) .
 		    '" src="../images/circle.png" /> ' . htmlspecialchars($row['probid']) . '</td>';
 
-		// team name, room and category
+		// team name, location (room) and category
 		echo '<td class="teamid">' . htmlspecialchars($row['login']) . '</td><td>' .
 			htmlspecialchars($row['teamname']) . '</td><td>' .
 			htmlspecialchars($row['room']) . '</td><td>' .
@@ -125,9 +122,10 @@ if ( !empty($BALLOONS) ) {
 
 		// list of balloons for this team
 		sort($TOTAL_BALLOONS[$row['login']]);
+		$TOTAL_BALLOONS[$row['login']] = array_unique($TOTAL_BALLOONS[$row['login']]);
 		foreach($TOTAL_BALLOONS[$row['login']] as $prob_solved) {
 			echo '<img title="' . htmlspecialchars($prob_solved) .
-				'" style="background-color: ' .
+				'" class="balloonimage" style="background-color: ' .
 				htmlspecialchars($probs_data[$prob_solved]['color']) .
 				';" alt="problem colour ' .
 				htmlspecialchars($probs_data[$row['probid']]['color']) .
@@ -136,12 +134,26 @@ if ( !empty($BALLOONS) ) {
 		echo '</td><td>';
 
 		// 'done' button when balloon has yet to be handed out
-		if ( $row['balloon'] == 0 ) {
+		if ( $row['done'] == 0 ) {
 			echo '<input type="submit" name="done[' .
-				htmlspecialchars($row['probid']) . ';' .
-				htmlspecialchars($row['teamid']) . ';' .
-				htmlspecialchars($row['cid']) . ']" value="done" />';
+				(int)$row['balloonid'] . ']" value="done" />';
 		}
+		
+		echo '</td><td>';
+
+		$comments = array();
+		if ( $first_contest == $row['balloonid'] ) {
+			$comments[] = 'first in contest';
+		} else {
+			if ( $first_team[$row['login']] == $row['balloonid'] ) {
+				$comments[] = 'first for team';
+			}	
+			if ( $first_problem[$row['probid']] == $row['balloonid'] ) {
+				$comments[] = 'first for problem';
+			}
+		}
+		echo implode('; ', $comments);
+
 		echo "</td></tr>\n";
 	}
 
