@@ -2,8 +2,6 @@
 /**
  * View of one contest.
  *
- * $Id: contest.php 3652 2011-10-08 09:57:29Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -11,7 +9,7 @@
 $id = (int)@$_GET['id'];
 
 require('init.php');
-$title = "Contest: " .htmlspecialchars(@$id);
+$title = "Contest";
 
 require(LIBWWWDIR . '/header.php');
 
@@ -37,20 +35,20 @@ if ( IS_ADMIN && !empty($_GET['cmd']) ):
 
 <tr><td><label for="data_0__contestname_">Contest name:</label></td>
 <td><?php echo addInput('data[0][contestname]', @$row['contestname'], 40, 255)?></td></tr>
-<tr><td><label for="data_0__activatetime_">Activate time:</label></td>
-<td><?php echo addInput('data[0][activatetime]', @$row['activatetime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> -hh:mm)</td></tr>
+<tr><td><label for="data_0__activatetime_string_">Activate time:</label></td>
+<td><?php echo addInput('data[0][activatetime_string]', @$row['activatetime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> -hh:mm)</td></tr>
 
 <tr><td><label for="data_0__starttime_">Start time:</label></td>
 <td><?php echo addInput('data[0][starttime]', @$row['starttime'], 20, 19)?> (yyyy-mm-dd hh:mm:ss)</td></tr>
 
-<tr><td><label for="data_0__freezetime_">Scoreboard freeze time:</label></td>
-<td><?php echo addInput('data[0][freezetime]', @$row['freezetime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> +hh:mm)</td></tr>
+<tr><td><label for="data_0__freezetime_string_">Scoreboard freeze time:</label></td>
+<td><?php echo addInput('data[0][freezetime_string]', @$row['freezetime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> +hh:mm)</td></tr>
 
-<tr><td><label for="data_0__endtime_">End time:</label></td>
-<td><?php echo addInput('data[0][endtime]', @$row['endtime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> +hh:mm)</td></tr>
+<tr><td><label for="data_0__endtime_string_">End time:</label></td>
+<td><?php echo addInput('data[0][endtime_string]', @$row['endtime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> +hh:mm)</td></tr>
 
-<tr><td><label for="data_0__unfreezetime_">Scoreboard unfreeze time:</label></td>
-<td><?php echo addInput('data[0][unfreezetime]', @$row['unfreezetime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> +hh:mm)</td></tr>
+<tr><td><label for="data_0__unfreezetime_string_">Scoreboard unfreeze time:</label></td>
+<td><?php echo addInput('data[0][unfreezetime_string]', @$row['unfreezetime_string'], 20, 19)?> (yyyy-mm-dd hh:mm:ss <i>or</i> +hh:mm)</td></tr>
 
 <tr><td>Enabled:</td><td>
 <?php echo addRadioButton('data[0][enabled]', (!isset($row['enabled']) ||  $row['enabled']), 1)?> <label for="data_0__enabled_1">yes</label>
@@ -75,11 +73,11 @@ if ( ! $id ) error("Missing or invalid contest id");
 
 if ( isset($_GET['edited']) ) {
 
-	echo addForm('refresh_cache.php', 'get') .
+	echo addForm('refresh_cache.php') .
             msgbox (
                 "Warning: Refresh scoreboard cache",
-		"After changing contest times, it may be necessary to recalculate the cached scoreboards.<br /><br />" .
-		addSubmit('recalculate caches now') 
+		"If the contest start time was changed, it may be necessary to recalculate any cached scoreboards.<br /><br />" .
+		addSubmit('recalculate caches now', 'refresh') 
 		) .
 		addEndForm();
 
@@ -91,7 +89,7 @@ $data = $DB->q('TUPLE SELECT * FROM contest WHERE cid = %i', $id);
 echo "<h1>Contest: ".htmlspecialchars($data['contestname'])."</h1>\n\n";
 
 if ( $cid == $data['cid'] ) {
-	echo "<p><em>This is the current contest.</em></p>\n\n";
+	echo "<p><em>This is the active contest.</em></p>\n\n";
 }
 if ( !$data['enabled'] ) {
 	echo "<p><em>This contest is disabled.</em></p>\n\n";
@@ -121,6 +119,9 @@ echo '<tr><td scope="row">Scoreboard unfreeze:</td><td>' .
 echo "</table>\n\n";
 
 if ( IS_ADMIN ) {
+	if ( $cid == $data['cid'] ) {
+		echo "<p>". rejudgeForm('contest', $data['cid']) . "</p>\n\n";
+	}
 	echo "<p>" .
 		editLink('contest',$data['cid']) . "\n" .
 		delLink('contest','cid',$data['cid']) ."</p>\n\n";
