@@ -2,8 +2,6 @@
 /**
  * View a problem
  *
- * $Id: problem.php 3459 2010-11-28 16:50:32Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -32,11 +30,13 @@ if ( !empty($pcmd) ) {
 	if ( isset($pcmd['toggle_submit']) ) {
 		$DB->q('UPDATE problem SET allow_submit = %i WHERE probid = %s',
 			   $_POST['val']['toggle_submit'], $id);
+		auditlog('problem', $id, 'set allow submit', $_POST['val']['toggle_submit']);
 	}
 
 	if ( isset($pcmd['toggle_judge']) ) {
 		$DB->q('UPDATE problem SET allow_judge = %i WHERE probid = %s',
 			   $_POST['val']['toggle_judge'], $id);
+		auditlog('problem', $id, 'set allow judge', $_POST['val']['toggle_judge']);
 	}
 }
 if ( isset($_POST['upload']) ) {
@@ -45,6 +45,7 @@ if ( isset($_POST['upload']) ) {
 		$zip = openZipFile($_FILES['problem_archive']['tmp_name']);
 		$id = importZippedProblem($zip, empty($id) ? NULL : $id);
 		$zip->close();
+		auditlog('problem', $id, 'upload zip', $_FILES['problem_archive']['name']);
 		header('Location: '.$pagename.'?id='.urlencode($id));
 	} else {
 		error("Missing filename for problem upload");
@@ -110,7 +111,7 @@ echo addSelect('data[0][cid]', $cmap, @$row['cid'], true);
 <td><?php echo addInputField('text','data[0][color]', @$row['color'],
 	' size="8" maxlength="25" class="color {required:false,adjust:false,hash:true,caps:false}"')?>
 <a target="_blank"
-href="http://www.w3schools.com/css/css_colornames.asp"><img
+href="http://www.w3schools.com/cssref/css_colornames.asp"><img
 src="../images/b_help.png" class="smallpicto" alt="?" /></a></td></tr>
 
 <tr><td><label for="data_0__special_run_">Special run script:</label></td>
@@ -215,7 +216,10 @@ echo "</table>\n" . addEndForm();
 echo "<br />\n" . rejudgeForm('problem', $id) . "\n\n";
 
 if ( IS_ADMIN ) {
-	echo "<p>" . editLink('problem',$id) . "\n" .
+	echo "<p>" . 
+		'<a href="export.php?id=' . urlencode($id) .
+		'"><img src="../images/b_save.png" /></a> ' .
+		editLink('problem',$id) . "\n" .
 		delLink('problem','probid', $id) . "</p>\n\n";
 }
 
