@@ -2,8 +2,6 @@
 /**
  * View team details
  *
- * $Id: team.php 3682 2011-10-11 15:20:50Z kink $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -60,17 +58,20 @@ echo addSelect('data[0][categoryid]', $cmap, @$row['categoryid'], true);
 <td><?php echo addTextArea('data[0][members]', @$row['members'], 40, 3)?></td></tr>
 <tr><td><label for="data_0__affilid_">Affiliation:</label></td>
 <td><?php
-$amap = $DB->q("KEYVALUETABLE SELECT affilid,name FROM team_affiliation ORDER BY affilid");
+$amap = $DB->q("KEYVALUETABLE SELECT affilid,name FROM team_affiliation ORDER BY name");
 $amap[''] = 'none';
 echo addSelect('data[0][affilid]', $amap, @$row['affilid'], true);
 ?>
 </td></tr>
 <tr><td><label for="data_0__authtoken_">Auth token:</label></td>
 <td><?php echo addInput('data[0][authtoken]', @$row['authtoken'], 35, 255)?></td></tr>
-<tr><td><label for="data_0__room_">Room:</label></td>
+<tr><td><label for="data_0__room_">Location:</label></td>
 <td><?php echo addInput('data[0][room]', @$row['room'], 10, 15)?></td></tr>
 <tr><td><label for="data_0__comments_">Comments:</label></td>
 <td><?php echo addTextArea('data[0][comments]', @$row['comments'])?></td></tr>
+<tr><td><label for="data_0__enabled_">Enabled:</label></td>
+<td><?php echo addRadioButton('data[0][enabled]', (!isset($row['']) || $row['enabled']), 1)?> <label for="data_0__enabled_1">yes</label>
+<?php echo addRadioButton('data[0][enabled]', (isset($row['enabled']) && !$row['enabled']), 0)?> <label for="data_0__enabled_0">no</label></td></tr>
 </table>
 
 <?php
@@ -107,6 +108,10 @@ $teamimage   = "../images/teams/"        . urlencode($row['login'])   . ".jpg";
 
 echo "<h1>Team ".htmlspecialchars($row['name'])."</h1>\n\n";
 
+if ( $row['enabled'] != 1 ) {
+	echo "<p><em>Team is disabled</em></p>\n\n";
+}
+
 ?>
 
 <div class="col1"><table>
@@ -115,7 +120,7 @@ echo "<h1>Team ".htmlspecialchars($row['name'])."</h1>\n\n";
 <tr><td scope="row">Host:</td><td><?php echo
 	(@$row['hostname'] ? printhost($row['hostname'], TRUE):'') ?></td></tr>
 <?php if (!empty($row['room'])): ?>
-<tr><td scope="row">Room:</td><td><?php echo htmlspecialchars($row['room'])?></td></tr>
+<tr><td scope="row">Location:</td><td><?php echo htmlspecialchars($row['room'])?></td></tr>
 <?php endif; ?>
 </table></div>
 
