@@ -2,8 +2,6 @@
 /**
  * Include required files.
  *
- * $Id: init.php 3575 2011-07-30 11:28:24Z eldering $
- *
  * Part of the DOMjudge Programming Contest Jury System and licenced
  * under the GNU GPL. See README and COPYING for details.
  */
@@ -26,7 +24,7 @@ require_once(LIBDIR . '/use_db.php');
 parseLangExts();
 
 set_exception_handler('exception_handler');
-setup_database_connection('team');
+setup_database_connection();
 
 require_once(LIBWWWDIR . '/common.php');
 require_once(LIBWWWDIR . '/print.php');
@@ -39,6 +37,10 @@ if ( @$_POST['cmd']=='login' ) do_login();
 
 if ( !logged_in() ) show_loginpage();
 
+if ( $teamdata['enabled'] != 1 ) {
+	error("Team is not enabled.");
+}
+
 $cdata = getCurContest(TRUE);
 $cid = $cdata['cid'];
 
